/*
 *  FIPS-197 compliant AES implementation
 *
 *  Copyright (C) 2001-2003  Christophe Devine
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "aes.h"
#include "print.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "sort.h"

/*
 * Rijndael Monte Carlo Test: ECB mode
 * source: NIST - rijndael-vals.zip
 */

static void xor(unsigned char *key2, unsigned char *key1, unsigned char *xorval)
{
	int i;
	for (i = 0; i < 32; i++)
		key2[i] = key1[i] ^ xorval[i];
}

extern double data_test(unsigned char *data, int len);

#define DATA_SIZE 128
#define KEY_SIZE 128
#define NUM_RUNS 1000000

int main( void )
{
	int count = 0, i, j;
	aes_context ctx1;
	aes_context ctx2;
	unsigned char buf[16];
	unsigned char out1[16];
	unsigned char out2[16];
	unsigned char xorval[32];
	unsigned char keystart[32];
	unsigned char key1[32];
	unsigned char key2[32];

	unsigned char test_data[DATA_SIZE];
	unsigned char sorted[DATA_SIZE];
	unsigned char _data[DATA_SIZE];
    unsigned char *test;
    char rstate[] = "initialise3";
    int num_samples = 0;

    initstate(10, rstate, strlen(rstate));

	memset( buf, 0x00, 16 );
	memset( keystart, 0x00, 16 );
	memset( xorval, 0x10, 16 );

	for (j = 0; j < NUM_RUNS; j++)
	{
		double pval;
        num_samples += 1;
		for (i = 0; i < sizeof(test_data); i += KEY_SIZE/8)
		{
#if 0
			int kn = i + (j*sizeof(test_data));

			memcpy(key1, (char *)&kn, sizeof(kn));
#else
            int r = random() % KEY_SIZE;
			memcpy(key1, keystart, KEY_SIZE/8);
			memcpy(key2, keystart, KEY_SIZE/8);
            bit_xor(key1, r);
#endif
			//xor(key2, key1, xorval);

			aes_set_key( &ctx1, key1, KEY_SIZE );
			aes_set_key( &ctx2, key2, KEY_SIZE );

			aes_encrypt( &ctx1, buf, out1 );
			aes_encrypt( &ctx2, buf, out2 );

			/*memcpy(&test_data[i], out1, sizeof(out1));*/
			xor(buf, buf, out1);
			//memcpy(buf, out1, sizeof(out1));
			memcpy(&_data[i], out1, sizeof(out1));
			xor(&test_data[i], out1, out2);
		}
        test = test_data;
		pval = data_test(test, sizeof(test_data));
        add_sample(pval);
        if (j == 0)
        {
            memcpy(sorted, test, sizeof(test_data));
            sort(sorted, 0, sizeof(test_data));
            save_data(count, sorted, sizeof(test_data));
        }
		if (pval < 0.0001)
		{
			printf("%d %.9f %d\n",
				j, pval, num_bits(test, sizeof(test_data)));
			count += 1;
            if (pval < 0.000001)
            {
                memcpy(sorted, test, sizeof(test_data));
                sort(sorted, 0, sizeof(test_data));
                print_data(sorted, sizeof(test_data));
                save_data(count, sorted, sizeof(test_data));
            }
		}
	}

	printf("%d\n", count);

    print_histogram(num_samples);
    
    return( 0 );
}

